import React from 'react'
const {useReducer, useMemo, useRef, useEffect, useCallback} = React

function _ref(prev, action) {
  return {
    status: action.status,
    value: action.status === 'success' ? action.value : void 0,
    error: action.status === 'error' ? action.error : void 0,
  }
}

export function useAsync(asyncCallback, dependencies = []) {
  const cancelled = useRef(false)
  const [state, dispatch_] = useReducer(_ref, {
    status: 'idle',
    value: void 0,
    error: void 0,
  })

  const dispatch = (action) => !cancelled.current && dispatch_(action) // Cancel any pending async stuff on unmount

  function _ref2() {
    cancelled.current = true
  }

  useEffect(() => _ref2, []) // Creates a stable callback that manages our loading/success/error status updates
  // as the callback is invoked.

  const callback = useCallback(
    async (...args) => {
      cancelled.current = false
      dispatch({
        status: 'loading',
      })

      try {
        dispatch({
          status: 'success',
          value: await asyncCallback(...args),
        })
      } catch (error) {
        dispatch({
          status: 'error',
          error,
        })
      }
    }, // eslint-disable-next-line react-hooks/exhaustive-deps
    dependencies
  )

  function _ref3() {
    dispatch({
      status: 'cancelled',
    })
    cancelled.current = true
  }

  return [
    useMemo(
      () =>
        Object.assign(
          {
            cancel: _ref3,
          },
          state
        ),
      [state]
    ),
    callback,
  ]
}
export function useAsyncEffect(asyncCallback, dependencies) {
  const [state, callback] = useAsync(
    asyncCallback,
    dependencies || [Math.random()]
  ) // Runs the callback each time the dependencies change and cancels any
  // pending callbacks that were running previously

  useEffect(
    () => {
      callback()
      return state.cancel
    }, // eslint-disable-next-line react-hooks/exhaustive-deps
    dependencies
  )
  return state
}
